/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.farm;

import java.io.IOException;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.farm.model.FarmManager;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ProjectFactory;
import org.netbeans.spi.project.ProjectState;
import org.openide.filesystems.FileObject;
import org.openide.util.Lookup;

/**
 *
 * @author Timon Veenstra
 */
@org.openide.util.lookup.ServiceProvider(service = ProjectFactory.class)
public class FarmProjectFactory implements ProjectFactory {

    //Specifies when a project is a project, i.e.,
    //if farm binary file is present:
    @Override
    public boolean isProject(FileObject projectDirectory) {
        return projectDirectory.getFileObject(FarmProject.FARM_FILE) != null;
    }

    //Specifies when the project will be opened, i.e.,
    //if the project exists:
    @Override
    public Project loadProject(FileObject dir, ProjectState state) throws IOException {
        FarmProject project = null;
        if (isProject(dir)) {
            project = new FarmProject(dir, state);
        }
        return project;
    }

    @Override
    public void saveProject(final Project project) throws IOException, ClassCastException {
        
        FarmManager farmManager = Lookup.getDefault().lookup(FarmManager.class);
        Farm farm = project.getLookup().lookup(Farm.class);
        farmManager.save(farm);
    }
}
