/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.farm;

import java.awt.event.ActionEvent;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import javax.swing.AbstractAction;
import nl.cloudfarming.client.model.EntitySavable;
import nl.cloudfarming.client.model.FarmID;
import org.netbeans.api.actions.Savable;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.util.NbBundle;

/**
 *
 * @author johan
 */
@ActionID(category = "File", id = "nl.cloudfarming.client.farm.FarmProjectPendingChangesAction")
@ActionRegistration(iconBase = "nl/cloudfarming/client/farm/pending_changes.png",
        displayName = "#CTL_FarmProjectPendingChangesAction")
//@ActionReferences({
//    @ActionReference(path = "Menu/File", position = 700),
//    @ActionReference(path = "FarmProjects/Actions")
//})
@NbBundle.Messages({"CTL_FarmProjectPendingChangesAction=Show pending changes"})
public class FarmProjectPendingChangesAction extends AbstractAction {
    List<FarmProject> projects;

    public FarmProjectPendingChangesAction(List<FarmProject> projects) {
        this.projects = projects;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        Set<FarmID> farmIDs = new HashSet<>();
        Set<EntitySavable> savables = new HashSet<>();

        for (FarmProject project : projects) {
            farmIDs.add(project.getFarmID());
        }
        
        // collect outstanding changes for selected projects:
        for (EntitySavable savable : Savable.REGISTRY.lookupAll(EntitySavable.class)) {
            if (farmIDs.contains(savable.getFarmID())) {
                savables.add(savable);
            }
        }
        
        if (!savables.isEmpty()) {
            FarmProjectSaveDialog.showDialog(savables);
            // FIXME: discard option needs to be hidden or behave differently from the "close project" version (e.g. Undo).
        } else {
            // TODO: notify or log message "no pending changes"
        }


    }

}
