/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.farm;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JToolBar;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.farm.model.FarmDetailsController;
import nl.cloudfarming.client.farm.model.FarmDetailsEditPanel;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;
import org.openide.windows.TopComponent;

@MultiViewElement.Registration(displayName = "#LBL_Farm_PROPERTIES",
iconBase = "nl/cloudfarming/client/farm/house.png",
mimeType = "application/x-agrosense-farm",
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "FarmVisual",
position = 100)
@Messages({"LBL_Farm_PROPERTIES=Properties",
    "FarmVisualElement name=FarmVisualElement",
    "FarmVisualElement invalid message=There are some errors please correct them"})
public final class FarmVisualElement extends FarmDetailsController<FarmDetailsEditPanel> implements MultiViewElement, PropertyChangeListener {

    private JToolBar toolbar = new JToolBar();
    private transient MultiViewElementCallback callback;
    private Lookup lkp;

    public FarmVisualElement(Lookup lkp) {
        super(new FarmDetailsEditPanel(lkp.lookup(Farm.class)));
        this.lkp = lkp;
        getLookup().lookup(Farm.class).addPropertyChangeListener(this);
    }

    @Override
    public JComponent getVisualRepresentation() {
        return super.getPanel();
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return lkp;
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    public void updateName() {
        if (callback != null) {
            String name = getLookup().lookup(Farm.class).getName();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
            tc.setName(Bundle.FarmVisualElement_name());
            tc.setToolTipText(Bundle.FarmVisualElement_name());
        }
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if (Farm.PROP_NAME.equals(evt.getPropertyName())) {
            updateName();
        }
    }

    @Override
    public void farmDetailFieldsUpdated() {
        if (super.validate()) {
            farmDetailPanel.getErrorPanel().setValid();
        } else {
            farmDetailPanel.getErrorPanel().setError(Bundle.FarmVisualElement_invalid_message());
        }
    }
}
