/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.farm;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Set;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.farm.model.FarmDetailsController;
import nl.cloudfarming.client.model.Address;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.IssuingAgency;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/**
 *
 * @author Timon Veenstra
 */
@NbBundle.Messages({
    "FarmWizardDetailsPanel invalid message=There are some errors please correct them",
    "FarmWizardDetailsPanel invalid enterprise id=The combination issuing agency / registration number must be unique",
})
public class FarmWizardDetailsPanel extends FarmDetailsController<FarmWizardDetailsVisual> implements WizardDescriptor.Panel,
        WizardDescriptor.FinishablePanel {

    private WizardDescriptor wizardDescriptor;
    private final Set<ChangeListener> listeners = new HashSet<>(1); // or can use ChangeSupport in NB 6.0

    public FarmWizardDetailsPanel() {
        super(new FarmWizardDetailsVisual(new Address()));
        farmDetailPanel.getFarmIssuingComboBox().addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                farmDetailFieldsUpdated();
            }
        });
    }

    @Override
    public Component getComponent() {
        return super.getPanel();
    }

    @Override
    public HelpCtx getHelp() {
        return new HelpCtx(FarmWizardDetailsPanel.class);
    }

    @Override
    public void farmDetailFieldsUpdated() {
        this.fireChangeEvent();
    }

    @Override
    public boolean isValid() {

        boolean valid = super.validate();
        String errorMsg = "";
        
        // check for unique URI:
        if (valid && !isUniqueEnterpriseId()) {
            valid = false;
            farmDetailPanel.getRegistrationNumberRow().setInvalid(Bundle.FarmWizardDetailsPanel_invalid_enterprise_id());
        }
        
        if (!valid) {
            errorMsg = Bundle.FarmWizardDetailsPanel_invalid_message();
        }
        wizardDescriptor.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, errorMsg);
        return valid;
    }
    
    /**
     * Check whether the combination of issuing agency and registration 
     * number is unique across all registered projects.
     * 
     * @return 
     */
    protected boolean isUniqueEnterpriseId() {
        EnterpriseIdProvider eip = new EnterpriseIdProvider() {
            @Override
            public IssuingAgency getIssuingAgency() {
                return farmDetailPanel.getFarmIssuingAgency();
            }
            @Override
            public String getEnterpriseId() {
                return farmDetailPanel.getFarmRegistrationNumber();
            }
        };
        
        // check open farm projects:
//        return EnterpriseIdProvider.ProjectFinder.find(eip) == null;
        // check registered farm projects:
        return !FarmProjectRegistry.getInstance().contains(eip);
    }
    
    @Override
    public final void addChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.add(l);
        }
    }

    @Override
    public final void removeChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.remove(l);
        }
    }

    protected final void fireChangeEvent() {
        Set<ChangeListener> ls;
        synchronized (listeners) {
            ls = new HashSet<>(listeners);
        }
        ChangeEvent ev = new ChangeEvent(this);
        for (ChangeListener l : ls) {
            l.stateChanged(ev);
        }
    }

    @Override
    public void readSettings(Object settings) {
        wizardDescriptor = (WizardDescriptor) settings;

        String name = (String) wizardDescriptor.getProperty(FarmWizardIterator.PROP_PROJECT_NAME);
        if (name == null) {
            name = Bundle.Farm_default_name();
        }
        farmDetailPanel.setFarmName(name);

        IssuingAgency issuingAgency = (IssuingAgency) wizardDescriptor.getProperty(Farm.PROP_ISSUING_AGENCY);
        farmDetailPanel.setFarmIssuingAgency(issuingAgency);

        String registrationNumber = (String) wizardDescriptor.getProperty(Farm.PROP_REGISTRATIONNUMBER);
        farmDetailPanel.setFarmRegistrationNumber(registrationNumber);
    }

    @Override
    public void storeSettings(Object settings) {
        wizardDescriptor.putProperty(Farm.PROP_ISSUING_AGENCY, farmDetailPanel.getFarmIssuingAgency());
        wizardDescriptor.putProperty(Farm.PROP_REGISTRATIONNUMBER, farmDetailPanel.getFarmRegistrationNumber());
        wizardDescriptor.putProperty(Farm.PROP_ADDRESS, farmDetailPanel.getAddress());
    }

    @Override
    public boolean isFinishPanel() {
        return true;
    }
}
