/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.farm;

import java.awt.Component;
import java.util.HashSet;
import java.util.Set;
import javax.swing.ImageIcon;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.WizardDescriptor;
import org.openide.WizardValidationException;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * Panel just asking for basic info.
 */
@NbBundle.Messages({"Farm location title=Farm location"})
public class FarmWizardLocationPanel implements WizardDescriptor.Panel,
        WizardDescriptor.ValidatingPanel {

    private WizardDescriptor wizardDescriptor;
    private FarmWizardLocationVisual bodyPanel;
    private ASBodyPanel component;
    private final ImageIcon icon = ImageUtilities.loadImageIcon("nl/cloudfarming/client/farm/folder-house32.png", false);

    public FarmWizardLocationPanel() {
    }

    @Override
    public Component getComponent() {
        if (bodyPanel == null) {
            bodyPanel = new FarmWizardLocationVisual(this);
            component = new ASBodyPanel(bodyPanel);
            component.setName(Bundle.LBL_FarmDetailsStep());
            component.setIcon(icon);
            component.setTitle(Bundle.Farm_location_title());
        }
        return component;
    }

    @Override
    public HelpCtx getHelp() {
        return new HelpCtx(FarmWizardLocationPanel.class);
    }

    @Override
    public boolean isValid() {
        getComponent();
        return bodyPanel.valid(wizardDescriptor);
    }
    private final Set<ChangeListener> listeners = new HashSet<>(1); // or can use ChangeSupport in NB 6.0

    @Override
    public final void addChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.add(l);
        }
    }

    @Override
    public final void removeChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.remove(l);
        }
    }

    protected final void fireChangeEvent() {
        Set<ChangeListener> ls;
        synchronized (listeners) {
            ls = new HashSet<>(listeners);
        }
        ChangeEvent ev = new ChangeEvent(this);
        for (ChangeListener l : ls) {
            l.stateChanged(ev);
        }
    }

    @Override
    public void readSettings(Object settings) {
        wizardDescriptor = (WizardDescriptor) settings;
        bodyPanel.read(wizardDescriptor);
    }

    @Override
    public void storeSettings(Object settings) {
        WizardDescriptor d = (WizardDescriptor) settings;
        bodyPanel.store(d);
    }

    public boolean isFinishPanel() {
        return false;
    }

    @Override
    public void validate() throws WizardValidationException {
        getComponent();
        bodyPanel.validate(wizardDescriptor);
    }
}
