/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.farm;

import java.awt.Component;
import javax.swing.ImageIcon;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.model.Address;
import nl.cloudfarming.client.model.IssuingAgency;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * Summary Panel for the farm wizard
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages("FarmWizardSummaryPanel title=Summary")
public class FarmWizardSummaryPanel implements WizardDescriptor.Panel, WizardDescriptor.FinishablePanel {

    private ASBodyPanel panelDecorator;
    FarmWizardSummaryPanelVisual summaryPanelVisual = new FarmWizardSummaryPanelVisual();
    private ImageIcon titleIcon = new ImageIcon(ImageUtilities.loadImage("nl/cloudfarming/client/farm/house32.png"));
    private static final String HELP_ID = "nl.cloudfarming.client.farm.FarmWizardSummaryPanel";

    @Override
    public Component getComponent() {
        if (panelDecorator == null) {
            panelDecorator = new ASBodyPanel(summaryPanelVisual);
            panelDecorator.setIcon(titleIcon);
            panelDecorator.setTitle(Bundle.FarmWizardSummaryPanel_title());
            panelDecorator.setName(Bundle.FarmWizardSummaryPanel_title());
        }
        return panelDecorator;
    }

    @Override
    public HelpCtx getHelp() {
        return new HelpCtx(HELP_ID);
    }

    @Override
    public void readSettings(Object settings) {
        WizardDescriptor wizardDescriptor = (WizardDescriptor) settings;
        String farmName = (String) wizardDescriptor.getProperty(FarmWizardIterator.PROP_PROJECT_NAME);
        IssuingAgency agency = (IssuingAgency) wizardDescriptor.getProperty(Farm.PROP_ISSUING_AGENCY);
        Address address = (Address) wizardDescriptor.getProperty(Farm.PROP_ADDRESS);
        String registrationNumber = (String) wizardDescriptor.getProperty(Farm.PROP_REGISTRATIONNUMBER);

        summaryPanelVisual.getFarmNameValueLabel().setText(farmName);
        summaryPanelVisual.getIssuingAgencyValueLabel().setText(agency.name());
        summaryPanelVisual.getRegistrationNumberValueLabel().setText(registrationNumber);
        summaryPanelVisual.getAddresValueLabel().setText(getAddressLine(address));
        summaryPanelVisual.getPostalCodeValueLabel().setText(address.getPostalCode());
        summaryPanelVisual.getCityValueLabel().setText(address.getCity());
        if (address.getCountry() != null) {
            summaryPanelVisual.getCountryValueLabel().setText(address.getCountry().getName());
        }
    }

    /**
     * Takes an address object and returns a string displaying the street,
     * house number and house number addition contained by the given address
     * object.
     * @param address The address object you wish to display in a neat string
     * @return A string containing the street name, house number and addition of
     * the passed street object
     */
    private String getAddressLine(Address address) {
        StringBuilder addressStringBuilder = new StringBuilder();
        if (address.getStreetName() != null && !address.getStreetName().isEmpty()) {
            addressStringBuilder.append(address.getStreetName());
            addressStringBuilder.append(" ");
        }
        if (address.getHouseNumber() != null) {
            addressStringBuilder.append(address.getHouseNumber());
        }
        if (address.getAddition() != null && !address.getAddition().isEmpty()) {
            addressStringBuilder.append(address.getAddition());
        }

        return addressStringBuilder.toString();
    }

    @Override
    public void storeSettings(Object settings) {
    }

    @Override
    public boolean isValid() {
        return true;
    }

    @Override
    public void addChangeListener(ChangeListener l) {
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
    }

    @Override
    public boolean isFinishPanel() {
        return true;
    }
}
